#include "ESP8266.h"
#define LOG_ENABLED
#include "log.h"
#include "uart.h"
#include "systick.h"
#include <string.h>
#include <util/delay.h>

static char reply[64];
static uint8_t reply_pos;

static void uart_flush_rx(void)
{
	uint32_t end = systick_val() + 500;
	reply_pos = 0;
	reply[reply_pos] = '\0';
	do
	{	
		uint16_t rx = uart_getc();
		if ((rx >> 8) == 0)
		{

			reply[reply_pos++] = rx & 0xFF;
			reply[reply_pos] = '\0';			
			if ((rx & 0xFF) == '\r')
			{
				LOG("ESP RX: %s\n", reply);			
				reply_pos = 0;
				end = systick_val() + 500;
			}
			if (reply_pos == sizeof(reply)-1)
			{
				LOG("ESP RX: %s\n", reply);			
				reply_pos = 0;			
			}
		}
	}
	while(systick_val() < end);
}

uint8_t ESP8266_send_cmd(const char* cmd, uint8_t wait_ok)
{
	LOG("ESP TX: %s\n", cmd);
	uart_flush_rx();

	uart_puts(cmd);
	
	if (wait_ok)
	{
		return ESP8266_recv("OK\r\n", NULL);
	}

	return 0;
}

uint8_t ESP8266_recv(const char* reply_ok, const char* reply_nok)
{
	// try to receive reply
	uint32_t start = systick_val();
	reply_pos = 0;
	for(;;)
	{
		if (systick_val() - start >= 500UL)
		{
			// timeout;
			reply[reply_pos] = '\0';
			LOG("ESP RX (timeout): %s\n", reply);			
			return 1;
		}
		
		uint16_t rx = uart_getc();
		
		if ((rx >> 8) == 0)
		{
			// byte received
			reply[reply_pos++] = rx & 0xFF;
			reply[reply_pos] = '\0';			
			if (reply_pos == sizeof(reply)-1)
			{
				LOG("ESP RX (full): %s\n", reply);
				// RX buffer overflow
				reply_pos = 0;
			}
			if (reply_ok && strstr(reply, reply_ok))
			{
				LOG("ESP RX: %s\n", reply);
				return 0;
			}
			if (reply_nok && strstr(reply, reply_nok))
			{
				LOG("ESP RX: %s\n", reply);
				return 1;			
			}
		}
	}	
}

char* ESP8266_last_reply(void)
{
	return reply;
}
